<?php
/**
 * GetSimple plugin - Simply add contact forms in your pages
 *
 * This plugin let you add contact forms in your pages by writing simple tags.
 * You can also define recipients or create your own complex forms.
 *
 * Plugin page : http://nliautaud.fr/wiki/travaux/getsimple_p01-contact
 * Plugin repository : http://get-simple.info/extend/plugin/p01-contact/35
 * @author Nicolas Liautaud <contact@nliautaud.fr>
 * @package p01-contact
 * @version 0.6
 */
require_once 'p01-contact/config.php';

$thisfile = basename(__FILE__, '.php');

register_plugin(
	$thisfile, 	            // ID of plugin, should be filename minus php
	'p01-contact', 	        // Title of plugin
	'0.6', 		            // Version of plugin
	'Nicolas Liautaud',	    // Author of plugin
	'http://nliautaud.fr', 	// Author URL
	P01CTDESC,              // Plugin Description
	'plugins', 	            // Page type of plugin
	''  	                // Function that displays content
);

add_filter('content','p01contact');

/**
 * Parse a string and replace tags by forms
 *
 * Create forms data structure, check if form have been posted,
 * manage some errors and send email if necessary.
 * @param string $contents the string to parse
 * @return string the modified string
 */
function p01contact($contents)
{  
    global $p01contact_settings;
    $tmp_content = $contents;
    $parameters = $targets = $forms_data = array();
    
    // find tags
    $pattern = '`(?<!<code>)\(%\s*contact\s*:?\s*(.*)\s*%\)`';  
    preg_match_all($pattern, $tmp_content, $tags, PREG_SET_ORDER);
    
    // find tags parameters
    $param_pattern = '`([^ ,!]+)(!)?\s*(&quot;([^&]*)&quot;)?\s*((=&gt;|=)?\s*([^,]*))?\s*`';
    foreach($tags as $id => $tag) {
        $forms_data[$id] = array('targets' => array(),
                                 'parameters' => array());
        $tag[1] = trim(preg_replace('`&nbsp;`',' ', $tag[1]));
        preg_match_all($param_pattern, $tag[1], $params, PREG_SET_ORDER);
        
        foreach($params as $param) {
            if(check_field_content('email', $param[0])) {
                array_push($forms_data[$id]['targets'], $param[0]);
            } else {
                $parameter['name'] = $param[1];
                $parameter['required'] = $param[2] == '!' ? 1 : 0;
                $parameter['title'] = $param[4];
                $parameter['locked'] = $param[6] == '=&gt;' ? 1 : 0;
                $parameter['value'] = $param[7];
                $forms_data[$id]['parameters'][$param[1]] = $parameter;
            }
        }
        // if there is no recipients, use admin address
        if(empty($forms_data[$id]['targets'])){
            $userdata = getXML(GSDATAOTHERPATH . 'user.xml');
            $forms_data[$id]['targets'][0] = $userdata->EMAIL;
        }
        // if there is no parameters, use default settings
        if(empty($forms_data[$id]['parameters'])){
            $forms_data[$id]['parameters'] = $p01contact_settings['default_fields'];
        }
    }
    
    if(!empty($_POST['contact'])) {
        // store and check posted data
        $form_id = $_POST['contact_id'];
        $forms_data[$form_id]['fields'] = format_data($_POST['contact']);
        $forms_data[$form_id]['errors'] = check_form_content($forms_data[$form_id]);
        
        if(empty($forms_data[$form_id]['errors'])) {
            if($p01contact_settings['enable'] == false) {
                // plugin is disable
                $forms_data[$form_id]['status'] = array(
                    'sent' => false,
                    'message' => 'plugin_disable'
                );
            } elseif(empty($forms_data[$form_id]['targets'])) {
                // there is no target
                $forms_data[$form_id]['status'] = array(
                    'sent' => false,
                    'message' => 'no_target'
                );
            } else {
                // send mail
                $forms_data[$form_id]['status'] = send_mail($forms_data[$form_id]);
            }
        } else {
            $forms_data[$form_id]['status']['sent'] = false;
        }
    }
    
    // modify content
    foreach($tags as $id => $tag) {
        $form = add_form($id, $forms_data[$id]);
        $tmp_content = preg_replace($pattern, $form, $tmp_content, 1);
    }
    
    return $tmp_content;
}

/**
 * Check form fields values
 *
 * Check if field is empty and required or
 * not empty but not valid.
 * @param array $form_data the form datas
 * @return array the errors list
 */
function check_form_content($form_data)
{
    $error = array();
    foreach($form_data['fields'] as $name => $value) {
        if($value == null && $form_data['parameters'][$name]['required']) {
            // empty and required
            $error[$name] = 'required_field';
        }
        elseif($value != null && !check_field_content($name, $value)) {
            // not empty but not valid
            $error[$name] = 'error_' . $name;
        }
    }
    return $error;
}

function check_field_content($object, $value)
{
    switch($object) {
        case 'email':
            $pattern = '`^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$`i';
            if(preg_match($pattern, $value)) return 1;
            else return 0;
        case 'phone':
            $pattern = '`^\+?[-0-9(). ]{6,}$$`i';
            if(preg_match($pattern, $value)) return 1;
            else return 0;
        case 'website':
            $pattern = "`^((http|https|ftp):\/\/(www\.)?|www\.)[a-zA-Z0-9\_\-]+\.([a-zA-Z]{2,4}|[a-zA-Z]{2}\.[a-zA-Z]{2})(\/[a-zA-Z0-9\-\._\?\&=,'\+%\$#~]*)*$`i";
            if(preg_match($pattern, $value)) return 1;
            else return 0;
        case 'message':
            $size = strlen($value);
            if($size > 100) return 1;
            else return 0;
        case 'captcha':
            include_once P01CTCAPTCHA . 'securimage.php';
            $securimage = new Securimage();
            if ($securimage->check($value) == false) return 0;
            else return 1;   
        default:
            return 1;
    }
}

function field_required($name, $parameters) {
    global $p01contact_settings;
    if( in_array($name, $p01contact_settings['fields_required']) ||
        in_array($name.'!', $parameters)) {
        return true;
    } else {
        return false;
    }
}

/**
 * Return an html form based on form datas
 * @param int $id the form id (for hidden input and anchor)
 * @param array $form_datas the form datas
 * @return string
 */
function add_form($id, $form_data)
{
    global $p01contact_settings;

    $form  = '<form action="#p01-contact_' . $id . '" ';
    $form .= 'id="p01-contact_' . $id . '" class="p01-contact" method="post">';
    // display status message
    if(!empty($form_data['status']['message'])) {
        $class = $form_data['status']['sent'] ? 'updated' : 'error';
        $form .= '<div class="'.$class.'">' . lang($form_data['status']['message']) . '</div>';
    }
    // display settings fields or tag-defined fields ?
    if(!empty($form_data['parameters'])){
        $fields = $form_data['parameters'];
    } else {
        $fields = $p01contact_settings['fields_shown'];
    }
    // add fields
    foreach($form_data['parameters'] as $field) {
        $form .= '<div>' . add_field($field, $form_data) . '</div>';
    }
    $form .= '<div><input name="contact_id" type="hidden" value="' . $id . '" />';
    $form .= '<input class="submit" id="contact-submit" name="contact-submit" ';
    $form .= 'type="submit" value="' . lang('send') . '" /></div>';
    $form .= '</form>';
    
    return $form;
}

/**
 * Return an html form field
 *
 * The form datas are used to compare field
 * to known errors or precedent values.
 * @param array $field the field datas
 * @param array $form_datas the form datas
 * @return string
 */
function add_field($field, $form_data)
{
    global $p01contact_settings;
    foreach($field as $key => $val) $$key = $val;
    if(!$type = specialfield_type($field)){
        $type = $name;
    }

    $row  = '<label for="' . $name . '">';
    $row .= !empty($title) ? $title : lang($name);
    $row .= $required ? ' <strong>*</strong>' : '';
    $row .= '</label>';

    // fields error messages
    if(isset($form_data['errors'][$name])) {
        $row .= ' <span style="font-size:0.7em;color:red">';
        $row .= lang($form_data['errors'][$name]);
        $row .= '</span>';
    }

    // fill fields by precedent value
    if( isset($form_data['fields'][$name]) &&
        isset($form_data['status']['sent']) &&
        $form_data['status']['sent'] == false)
    {
        $value = $form_data['fields'][$name];
    }
    
    $row .= '<br/>';
    
    switch($type) {
        case 'textarea' :
            $row .= '<textarea id="' . $name . '" ';
            $row .=     'name="contact[' . $name . ']" rows="10"';
            $row .=     $locked ? ' disabled="disabled"' : '';
            $row .= '>' . $value . '</textarea>';
            break;
        case 'captcha' :
            $row .= '<img src="'.P01CTCAPTCHA.'securimage_show.php" ';
            $row .=     'alt="CAPTCHA Image" /><br/>';
            $row .= '<a href="#"';
            $row .=     'onclick="document.getElementById(\'captcha\').src = ';
            $row .=     P01CTCAPTCHA.'securimage_show.php?\' ';
            $row .=     '+ Math.random(); return false">';
            $row .=     lang('reload');
            $row .= '</a> ';
            $row .= '<input id="' . $name . '" ';
            $row .=     'type="text" name="contact[' . $name . ']" ';
            $row .=     'size="10" maxlength="6"';
            $row .=     $locked ? ' disabled="disabled" ' : ' ';
            $row .= ' />';
            break;
        case 'file' :
            $row .= '<input id="' . $name . '" ';
            $row .=     'name="contact[' . $name . ']" type="file"';
            $row .=     $locked ? ' disabled="disabled" ' : ' ';
            $row .= ' />';
            break;
        case 'checkbox' :
            $row .= '<input id="' . $name . '" ';
            $row .=     'name="contact[' . $name . ']" type="checkbox"';
            $row .=     $locked ? ' disabled="disabled" ' : ' ';
            $row .=     $value == 'on' ? 'checked' : '';
            $row .= ' />';
            break;
        case 'text' :
            $row .= '<input id="' . $name . '" ';
            $row .=     'name="contact[' . $name . ']" type="text" ';
            $row .=     'value="' . $value . '"';
            $row .=     $locked ? ' disabled="disabled" ' : ' ';
            $row .= ' />';
            break;
    }
    return $row;
}

/**
 * If the field is special, return its type
 * @param array $field the field datas
 * @return string|0
 */
function specialfield_type($field)
{
    global $p01contact_settings;
    if(isset($p01contact_settings['fields_types'][$field['name']])) {
        return $p01contact_settings['fields_types'][$field['name']];
    } else {
        return 0;
    }
}

/**
 * Send a mail based on form datas
 * @param array $form_data the form datas
 * @return array the sending result
 */
function send_mail($form_data)
{   
    $fields  = $form_data['fields'];
    $targets = implode(',', $form_data['targets']);
    $from    = $fields['email'];
    $subject = '=?utf-8?B?' . base64_encode($fields['subject']) . '?=';
    
    $content  = '<h1>' . lang('sentfrom') . ' ' . get_page_url(1) . '</h1>';
    $content .= '<ul><li>' . lang('date').' : <b>' . date('r') . '</b></li>';
    
    $skip = array('message', 'captcha');
    foreach($fields as $name => $value) {
        if(!in_array($name, $skip) && !empty($value)) {
            $content .= '<li>' . lang($name).' : <b>' . $value . '</b></li>';
        }
    }
    $content .= '</ul><p>' . nl2br($fields['message']) . '</p>';
    
    $headers  = "From: $from\r\n";
    $headers .= "Reply-To: $from\r\n";
    $headers .= "Return-Path: $from\r\n";
    $headers .= "MIME-Version: 1.0\r\n";
    $headers .= "Content-type: text/html; charset=UTF-8\r\n";
    $headers .= "X-Mailer: PHP/" . phpversion() . "\r\n" ;

    $sent = mail($targets, $subject, $content, $headers);
    if( isset($fields['askcopy']) &&
        $fields['askcopy'])
        $copy = mail($from, $subject, $content, $headers);
    
    if($sent) $status = array('sent' => true, 'message' => 'sent');
    else $status = array('sent' => false, 'message' => 'error_send');
    return $status;
}

/**
 * Format every array values (aesthetic and security)
 * @param array $array
 * @return array
 */
function format_data($array)
{
    foreach($array as $key => $val){
        $tmp = stripslashes($val);
        $tmp = htmlentities($tmp, ENT_QUOTES, 'UTF-8');
        $array[$key] = $tmp;
    }
    return $array;
}

/**
 * Return a traduction of the keyword
 
 * Manage languages between requested langs and existing traductions.
 * @param string $key the keyword
 * @return string
 */
function lang($key)
{
    global $p01contact_lang, $p01contact_settings, $LANG;
    
    $lang_prefix = substr($LANG, 0, 2);
    
    $lang = empty($p01contact_settings['lang']) ? $lang_prefix : $p01contact_settings['lang'];
    $lang = file_exists(P01CTLANG . $lang . '.php') ? $lang : 'en';
    
    include_once P01CTLANG . $lang . '.php';
    
    if(isset($p01contact_lang[$key])) {
        return $p01contact_lang[$key];
    } else {
        return ucfirst($key);
    }
}
